#!/bin/bash
#
#------------------------------------------------------------------------------
# Check input parameters (name of executable and new options)
#------------------------------------------------------------------------------
usage () {
   if [ -z "$rsync_em" ]; then echo "Usage: $0 <myrsync>"; exit 1; fi
   myrsync=`which $rsync_em 2> /dev/null | tail -1`
   if [ -z "$myrsync" ]; then echo "Non trovo $rsync_em!"; exit 1; fi
}
#------------------------------------------------------------------------------
# Funzione di inizializzazione dei tre alberi: sorgente e due destinazioni
#------------------------------------------------------------------------------
init_trees () {
   for dir in s0 t1 t2 
   do
      rm -r $dir 2> /dev/null
      mkdir $dir
      cd $dir
      tar -xvf ../jforc.tar > /dev/null
      cd ..
   done 
}
#------------------------------------------------------------------------------
# Crea le differenze negli alberi, classificate secondo il seguente schema:
#
# 1st char (Age): S/D/E/M (SourceOlder/DestOlder/Equal/MissingDest)
# 2nd char (Source): R/S/D (Regular/SymbolicLink/Directory)
# 3rd char (Destination): R/S/D (Regular/SymbolicLink/Directory)
#
# N.B.: i casi E/R/R, E/D/D, E/S/S sono coperti dalla creazione dei tre
#       alberi uguali
#------------------------------------------------------------------------------
mkdiff_trees () {
   cd s0/jforc			# modifiche all'albero sorgente
   touch w*htm								# D/R/R 
   echo "new line at the end." >> introduction.htm 			# D/R/R
   rm ln_loopless_code_vi.htm; touch ln_loopless_code_vi.htm		# D/S/R
   touch .hidden_file							# M/R/*
   cd ../..			# ritorno alla directory iniziale
   for dir in t1 t2 		# modifiche agli alberi destinazione
   do
      cd $dir/jforc
      rm valedictory.htm; ln -s introduction.htm valedictory.htm	# S/R/S
      rm declarations.htm; mkdir declarations.htm			# S/R/D
      head glossary.htm > glossary.htm					# S/R/R
      rm ln_loopless_code_iv.htm; touch ln_loopless_code_iv		# S/L/R
      rmdir empty_dir; touch empty_dir					# S/D/R
      rm ln_loopless_code_iiadverbs__and.htm				# M/S/*
      rm -r cdir							# M/D/*
      rm empty_operands.htm						# M/R/*
      cd ../..
   done
}
#------------------------------------------------------------------------------
# Test and check differences: the variables file1 and file2 
# contain the names of the file to check and scan simultanously the source and
# destination trees
#------------------------------------------------------------------------------
run() {
   local options=$@
   rsync     $options s0/ t1 > rsync.out     2> rsync.err
   $rsync_em $options s0/ t2 > $rsync_em.out 2> $rsync_em.err
   echo "File diversi creati da rsync e $rsync_em:"
   for file1 in `find t1 -print`	
   do
      file2=`echo $file1 | sed -e "s/^t1\//t2\//"` 	
      diff -q $file1 $file2 >> "$rsync_em.diff" 2>&1
      if [ $? -ne 0 ]; then echo -n "->"; basename $file1; fi
   done
   diff rsync.out $rsync_em.out >> "$rsync_em.outdiff"
   if [ $? -ne 0 ]; then echo "differenza in stdout!" 1>&2; fi
   diff rsync.err $rsync_em.err >> "$rsync_em.errdiff"
   if [ $? -ne 0 ]; then echo "differenza in stderr!" 1>&2; fi
}
#------------------------------------------------------------------------------
# Main program
#------------------------------------------------------------------------------
rsync_em=$1			# executable filename is first parameter
usage				# check input arguments
echo "Inizializzazione degli alberi sorgente e destinazione"
init_trees			# initializa source and desctination trees
echo "Inizializzazione delle differenze da verificare"
mkdiff_trees			# crea le differenze negli alberi
echo "Verifica del comportamento di $rsync_em con le opzioni -r e -u"
run -r -u      	 		# test! 
